package logic

import (
	"micro-bundle/internal/dao"
	"micro-bundle/internal/model"
	"micro-bundle/pb/bundle"
	"time"

	"github.com/dubbogo/gost/log/logger"
	"github.com/jinzhu/copier"
	"github.com/samber/lo"
)

func BundleExtend(req *bundle.BundleExtendRequest) (*bundle.BundleExtendResponse, error) {
	data := model.BundleExtensionRecords{}
	if err := copier.CopyWithOption(&data, req, copier.Option{DeepCopy: true}); err != nil {
		return nil, err
	}
	return nil, dao.AddBundleExtendRecord(data)
}

func BundleExtendRecordsList(req *bundle.BundleExtendRecordsListRequest) (*bundle.BundleExtendRecordsListResponse, error) {
	data, total, err := dao.GetBundleExtendRecordList(req)
	if err != nil {
		return nil, err
	}
	var result = &bundle.BundleExtendRecordsListResponse{}
	result.Data = lo.Map(data, func(data model.BundleExtendRecordItemPo, _ int) (ber *bundle.BundleExtendRecordItem) {
		ber = &bundle.BundleExtendRecordItem{}
		ber.CreatedAt = uint64(data.CreatedAt.UnixMilli())
		ber.UserName = data.UserName
		ber.UserPhoneNumber = data.UserPhoneNumber
		if data.ServiceType == 0 {
			ber.AccountAdditional += uint32(data.AccountAdditional)
			ber.VideoAdditional += uint32(data.VideoAdditional)
			ber.ImagesAdditional += uint32(data.ImagesAdditional)
			ber.DataAdditional += uint32(data.DataAdditional)
			ber.OperatorName = data.OperatorName
			ber.OperatorPhoneNumber = data.OperatorPhoneNumber
			ber.Type = 1
		} else {
			switch data.ServiceType {
			case 1:
				ber.VideoAdditional += uint32(data.ServiceNum)
			case 2:
				ber.ImagesAdditional += uint32(data.ServiceNum)
			case 3:
				ber.DataAdditional += uint32(data.ServiceNum)
			case 4:
				ber.AccountAdditional += uint32(data.ServiceNum)
			}
			ber.AssociatedOrderNumber = data.OrderUUID
			ber.Type = 2
		}
		return
	})
	result.Total = total
	return result, err
}

func GetBundleBalance(req *bundle.GetBundleBalanceReq) (*bundle.GetBundleBalanceResp, error) {
	// start := time.Now()
	// defer func() {
	// 	fmt.Printf("(time.Now().UnixMilli() - start.UnixMilli()): %v\n", (time.Now().UnixMilli() - start.UnixMilli()))
	// }()
	data, total, err := dao.GetBundleBalance(req)
	if err != nil {
		return nil, err
	}
	type userDataDto struct {
		UserID               int
		UserName             string
		UserPhoneNumber      string
		BundleName           string
		ExpirationTime       time.Time
		BundleStatus         int
		VideoCap             int
		ImageCap             int
		DataAnalysisCap      int
		AccountCap           int
		VideoUsed            int
		ImageUsed            int
		DataAnalysisUsed     int
		AccountUsed          int
		ExpansionPacksNumber int
	}
	usersMap := map[int]*userDataDto{}
	lo.ForEach(data, func(data model.BundleBalancePo, index int) {
		_, ok := usersMap[data.UserID]
		if !ok {
			usersMap[data.UserID] = &userDataDto{
				UserID:          data.UserID,
				UserName:        data.UserName,
				UserPhoneNumber: data.UserPhoneNumber,
				BundleStatus:    data.BundleStatus,
				ExpirationTime:  data.ExpirationTime,
			}
		}
	})
	for _, user := range usersMap {
		userData := lo.Filter(data, func(data model.BundleBalancePo, _ int) bool {
			return data.UserID == user.UserID
		})
		// 去重bundle_order_value_add.uuid 计算可用量
		lo.ForEach(lo.UniqBy(lo.Filter(userData, func(data model.BundleBalancePo, _ int) bool {
			return data.BovaUUID != nil
		}), func(data model.BundleBalancePo) string {
			return *data.BovaUUID
		}), func(data model.BundleBalancePo, index int) {
			if data.OderSource == 3 {
				user.ExpansionPacksNumber++
			}
			logger.Infof("增加数据类型%v(%v),user_id:%v,uuid:%v", data.ServiceType, data.ServiceNum, data.UserID, *data.BovaUUID)
			switch data.ServiceType {
			case 1:
				user.VideoCap += data.ServiceNum
			case 2:
				user.ImageCap += data.ServiceNum
			case 3:
				user.DataAnalysisCap += data.ServiceNum
			case 4:
				user.AccountCap += data.ServiceNum
			default:
				logger.Warn("不存在的数据类型:", data.ServiceType)
			}
		})

		// 去重work_category.uuid 统计绑定的发布数
		lo.ForEach(lo.UniqBy(lo.Filter(userData, func(data model.BundleBalancePo, _ int) bool {
			return data.CwUUID != nil
		}), func(data model.BundleBalancePo) string {
			return *data.CwUUID
		}), func(data model.BundleBalancePo, index int) {
			logger.Infof("减少数据类型%v,user_id:%v,uuid:%v", data.ServiceType, data.UserID, *data.CwUUID)
			switch data.WorkCategory {
			case 1:
				user.ImageUsed++
			case 2:
				user.VideoUsed++
			default:
				logger.Warn("不存在的数据类型:", data.WorkCategory)
			}
		})

		// 去重cast_media_account.uuid 统计绑定的账号数
		user.AccountUsed += len(lo.UniqBy(lo.Filter(userData, func(data model.BundleBalancePo, _ int) bool {
			return data.CmaUUID != nil
		}), func(data model.BundleBalancePo) string {
			return *data.CmaUUID
		}))

		// 去重bundle_extension_records.id 统计绑定的账号数
		lo.ForEach(lo.UniqBy(userData, func(data model.BundleBalancePo) int {
			return data.BerID
		}), func(data model.BundleBalancePo, index int) {
			user.ExpansionPacksNumber++
			user.AccountCap += data.AccountAdditional
			user.VideoCap += data.VideAadditiona
			user.DataAnalysisCap += data.DataAdditional
			user.ImageCap += data.ImagesAdditional
		})
	}
	result := &bundle.GetBundleBalanceResp{}
	result.Total = total
	result.Data = lo.MapToSlice(usersMap, func(_ int, user *userDataDto) *bundle.BundleBalanceItem {
		return &bundle.BundleBalanceItem{
			UserName:                      user.UserName,
			UserPhoneNumber:               user.UserPhoneNumber,
			BundleName:                    user.BundleName,
			Status:                        int32(user.BundleStatus),
			ExpiredTime:                   user.ExpirationTime.UnixMilli(),
			AccountNumber:                 int32(user.AccountCap),
			AccountConsumptionNumber:      int32(user.AccountUsed),
			ImageNumber:                   int32(user.ImageCap),
			ImageConsumptionNumber:        int32(user.ImageUsed),
			VideoNumber:                   int32(user.VideoCap),
			VideoConsumptionNumber:        int32(user.VideoUsed),
			DataAnalysisNumber:            int32(user.DataAnalysisCap),
			DataAnalysisConsumptionNumber: int32(user.DataAnalysisUsed),
			ExpansionPacksNumber:          int32(user.ExpansionPacksNumber),
		}
	})
	return result, nil
}
